@@ -24,7 +24,7 @@ use constant {
 
 sub __init {
     my ( $this, $feed, $req ) = @_;
-
+    
     if ( ref($feed) eq 'HASH' ) {
         $this->SUPER::__init( $feed, $req );
         $this->_media(
@@ -223,97 +223,104 @@ sub access_control {
         }
     }
     if ( @_ > 1 ) {
-
-        push @{ $this->_access_control },
-          new WebService::GData::YouTube::YT::AccessControl(
-            { action => $_[0], permission => $_[1] } );
+        #first check if the action is already set and if so update
+        my $ret = $this->_access_control->action( $_[0]  );
+        if ( @$ret > 0 ) {
+              $ret->[0]->permission( $_[1] );
+        }
+        #if not, just push a new entry;
+        else {
+              push @{ $this->_access_control },
+                new WebService::GData::YouTube::YT::AccessControl(
+                  { action => $_[0], permission => $_[1] } );
+        }
     }
     $this->_access_control;
 }
 
 sub delete {
-    my $this = shift;
-    my $uri  = $BASE_URI . $this->video_id;
-    $this->{_request}->delete( $uri, 0 );
+      my $this = shift;
+      my $uri  = $BASE_URI . $this->video_id;
+      $this->{_request}->delete( $uri, 0 );
 }
 
 sub save {
-    my ($this) = @_;
-    my $content = XML_HEADER . $this->serialize();
+      my ($this) = @_;
+      my $content = XML_HEADER . $this->serialize();
 
-    if ( $this->video_id ) {
+      if ( $this->video_id ) {
 
-        $this->{_request}->update( $BASE_URI . $this->video_id, $content );
-    }
-    else {
+         my $ret= $this->{_request}->update( $BASE_URI . $this->video_id, $content );
+      }
+      else {
 
-        if ( $this->upload_mode eq DIRECT_UPLOAD ) {
-            $this->direct_uploading( $UPLOAD_BASE_URI, $content );
-        }
-        else {
-            return $this->browser_uploading( $UPLOAD_BASE_URI, $content );
-        }
-    }
+          if ( $this->upload_mode eq DIRECT_UPLOAD ) {
+              $this->direct_uploading( $UPLOAD_BASE_URI, $content );
+          }
+          else {
+              return $this->browser_uploading( $UPLOAD_BASE_URI, $content );
+          }
+      }
 }
 
 #video upload
 
 sub filename {
-    my $this = shift;
-    return $this->{_filename} = $_[0] if ( @_ == 1 );
-    $this->{_filename};
+      my $this = shift;
+      return $this->{_filename} = $_[0] if ( @_ == 1 );
+      $this->{_filename};
 }
 
 #TODO: stream
 sub _binary_data {
-    my $this = shift;
+      my $this = shift;
 
-    if ( @_ == 1 ) {
-        my $fh = $_[0];
-        binmode($fh);
-        my $data = '';
-        while ( read $fh, my $buf, 1024 ) {
-            $data .= $buf;
-        }
-        close $fh;
-        return $this->{_binary_data} = $data;
-    }
-    $this->{_binary_data};
+      if ( @_ == 1 ) {
+          my $fh = $_[0];
+          binmode($fh);
+          my $data = '';
+          while ( read $fh, my $buf, 1024 ) {
+              $data .= $buf;
+          }
+          close $fh;
+          return $this->{_binary_data} = $data;
+      }
+      $this->{_binary_data};
 }
 
 sub upload_mode {
-    my $this = shift;
-    if ( @_ == 1 ) {
-        $this->{_UPLOAD_MODE} = shift;
-        $this->{_UPLOAD_MODE} = undef
-          if ( $this->{_UPLOAD_MODE} ne DIRECT_UPLOAD
-            || $this->{_UPLOAD_MODE} ne BROWSER_UPLOAD );
-    }
-    $this->{_UPLOAD_MODE} = BROWSER_UPLOAD if ( !$this->{_UPLOAD_MODE} );
-    $this->{_UPLOAD_MODE};
+      my $this = shift;
+      if ( @_ == 1 ) {
+          $this->{_UPLOAD_MODE} = shift;
+          $this->{_UPLOAD_MODE} = undef
+            if ( $this->{_UPLOAD_MODE} ne DIRECT_UPLOAD
+              || $this->{_UPLOAD_MODE} ne BROWSER_UPLOAD );
+      }
+      $this->{_UPLOAD_MODE} = BROWSER_UPLOAD if ( !$this->{_UPLOAD_MODE} );
+      $this->{_UPLOAD_MODE};
 }
 
 sub browser_uploading {
-    my ( $this, $uri, $content ) = @_;
-    my $response =
-      $this->{_request}
-      ->insert( 'http://gdata.youtube.com/action/GetUploadToken', $content );
+      my ( $this, $uri, $content ) = @_;
+      my $response =
+        $this->{_request}
+        ->insert( 'http://gdata.youtube.com/action/GetUploadToken', $content );
 
-    my ( $url, $token ) =
-      $response =~ m/<url>(.+?)<\/url><token>(.+?)<\/token>/;
-    if ( $this->next_url ) {
-        $url .= '?' . $this->next_url;
-    }
-    return ( $url, $token, $response );
+      my ( $url, $token ) =
+        $response =~ m/<url>(.+?)<\/url><token>(.+?)<\/token>/;
+      if ( $this->next_url ) {
+          $url .= '?' . $this->next_url;
+      }
+      return ( $url, $token, $response );
 }
 
-#move this in Base?
+#TODO:move this and rewrite!
 sub direct_uploading {
-    my ( $this, $uri, $content ) = @_;
+      my ( $this, $uri, $content ) = @_;
 
-    my $binary = $this->_binary_data;
+      my $binary = $this->_binary_data;
 
-    my $content2 = <<XML;
+      my $content2 = <<XML;
 
 --f93dcbA3
 Content-Type: application/atom+xml; charset=UTF-8
@@ -324,9 +331,9 @@ Content-Type: application/atom+xml; charset=UTF-8
   xmlns:yt="http://gdata.youtube.com/schemas/2007">
 XML
 
-    $content2 .= $content . '</entry>';
+      $content2 .= $content . '</entry>';
 
-    $content2 .= <<XML;
+      $content2 .= <<XML;
 
 --f93dcbA3
 Content-Type: video/quicktime
@@ -337,90 +344,90 @@ $binary
 --f93dcbA3--
 XML
 
-    my $req = HTTP::Request->new( POST => $uri );
+      my $req = HTTP::Request->new( POST => $uri );
 
-    if ( $this->{_request}->auth ) {
-        $this->{_request}->auth->set_authorization_headers( $this, $req );
-        $this->{_request}->auth->set_service_headers( $this, $req );
-    }
-    $req->header( 'GData-Version' => $this->{_request}->query->get('v') );
-    $req->header( 'Slug'          => $this->filename );
-    $req->content_type('multipart/related; boundary="f93dcbA3"');
-    $req->header( 'Content-Length' => length($content2) );
-    $req->header( 'Connection'     => 'close' );
-    $req->content($content2);
+      if ( $this->{_request}->auth ) {
+          $this->{_request}->auth->set_authorization_headers( $this, $req );
+          $this->{_request}->auth->set_service_headers( $this, $req );
+      }
+      $req->header( 'GData-Version' => $this->{_request}->query->get('v') );
+      $req->header( 'Slug'          => $this->filename );
+      $req->content_type('multipart/related; boundary="f93dcbA3"');
+      $req->header( 'Content-Length' => length($content2) );
+      $req->header( 'Connection'     => 'close' );
+      $req->content($content2);
 
-    my $res = $this->{_request}->{__UA__}->request($req);
-    if ( $res->is_success ) {
-        return $this;
-    }
-    else {
-        die new WebService::GData::Error( $res->code, $res->content );
-    }
+      my $res = $this->{_request}->{__UA__}->request($req);
+      if ( $res->is_success ) {
+          return $this;
+      }
+      else {
+          die new WebService::GData::Error( $res->code, $res->content );
+      }
 
 }
 
 sub is_listing_allowed {
-    my $this = shift;
-    if ( @_ == 1 ) {
-        $this->access_control->[0]->{permission} = $_[0];
-    }
-    return ( $this->access_control->[0]->{permission} eq 'allowed' ) ? 1 : 0;
+      my $this = shift;
+      if ( @_ == 1 ) {
+          $this->access_control->[0]->{permission} = $_[0];
+      }
+      return ( $this->access_control->[0]->{permission} eq 'allowed' ) ? 1 : 0;
 }
 
 sub is_comment_allowed {
-    my $this = shift;
-    if ( @_ == 1 ) {
-        $this->access_control->[1]->{permission} = $_[0];
-    }
-    return ( $this->access_control->[1]->{permission} eq 'allowed' ) ? 1 : 0;
+      my $this = shift;
+      if ( @_ == 1 ) {
+          $this->access_control->[1]->{permission} = $_[0];
+      }
+      return ( $this->access_control->[1]->{permission} eq 'allowed' ) ? 1 : 0;
 }
 
 sub is_comment_vote_allowed {
-    my $this = shift;
-    if ( @_ == 1 ) {
-        $this->access_control->[2]->{permission} = $_[0];
-    }
-    return ( $this->access_control->[2]->{permission} eq 'allowed' ) ? 1 : 0;
+      my $this = shift;
+      if ( @_ == 1 ) {
+          $this->access_control->[2]->{permission} = $_[0];
+      }
+      return ( $this->access_control->[2]->{permission} eq 'allowed' ) ? 1 : 0;
 }
 
 sub is_video_response_allowed {
-    my $this = shift;
-    if ( @_ == 1 ) {
-        $this->access_control->[3]->{permission} = $_[0];
-    }
-    return ( $this->access_control->[3]->{permission} eq 'allowed' ) ? 1 : 0;
+      my $this = shift;
+      if ( @_ == 1 ) {
+          $this->access_control->[3]->{permission} = $_[0];
+      }
+      return ( $this->access_control->[3]->{permission} eq 'allowed' ) ? 1 : 0;
 }
 
 sub is_rating_allowed {
-    my $this = shift;
-    if ( @_ == 1 ) {
-        $this->access_control->[4]->{permission} = $_[0];
-    }
-    return ( $this->access_control->[4]->{permission} eq 'allowed' ) ? 1 : 0;
+      my $this = shift;
+      if ( @_ == 1 ) {
+          $this->access_control->[4]->{permission} = $_[0];
+      }
+      return ( $this->access_control->[4]->{permission} eq 'allowed' ) ? 1 : 0;
 }
 
 sub is_embedding_allowed {
-    my $this = shift;
-    if ( @_ == 1 ) {
-        $this->access_control->[5]->{permission} = $_[0];
-    }
-    return ( $this->access_control->[5]->{permission} eq 'allowed' ) ? 1 : 0;
+      my $this = shift;
+      if ( @_ == 1 ) {
+          $this->access_control->[5]->{permission} = $_[0];
+      }
+      return ( $this->access_control->[5]->{permission} eq 'allowed' ) ? 1 : 0;
 }
 
 sub is_syndication_allowed {
-    my $this = shift;
-    if ( @_ == 1 ) {
-        $this->access_control->[6]->{permission} = $_[0];
-    }
-    return ( $this->access_control->[6]->{permission} eq 'allowed' ) ? 1 : 0;
+      my $this = shift;
+      if ( @_ == 1 ) {
+          $this->access_control->[6]->{permission} = $_[0];
+      }
+      return ( $this->access_control->[6]->{permission} eq 'allowed' ) ? 1 : 0;
 }
 
 private _urlencode => sub {
-    my ($string) = shift;
-    $string =~ s/(\W)/"%" . unpack("H2", $1)/ge;
-    return $string;
-};
+      my ($string) = shift;
+      $string =~ s/(\W)/"%" . unpack("H2", $1)/ge;
+      return $string;
+  };
 
 "The earth is blue like an orange.";
 
